/* Config.hh. Generated by CMake for ignition-fuel_tools5. */

/* Version number */
#define IGNITION_FUEL_TOOLS_MAJOR_VERSION 5
#define IGNITION_FUEL_TOOLS_MINOR_VERSION 1
#define IGNITION_FUEL_TOOLS_PATCH_VERSION 1

#define IGNITION_FUEL_TOOLS_VERSION "5.1"
#define IGNITION_FUEL_TOOLS_VERSION_FULL "5.1.1"

#define IGNITION_FUEL_TOOLS_VERSION_HEADER "ignition-fuel_tools5, version 5.1.1\nCopyright (C) 2017 Open Source Robotics Foundation.\nReleased under the Apache 2.0 License.\n\n"

#define IGNITION_FUEL_INITIAL_CONFIG_PATH "D:/bld/libignition-fuel-tools5_1613969207894/_h_env/Library/share/ignition/fuel_tools5/"

/* #undef BUILD_TYPE_PROFILE */
/* #undef BUILD_TYPE_DEBUG */
#define BUILD_TYPE_RELEASE 1
